@@ -3,7 +3,7 @@
 # Author: Natalia Bidart <natalia.bidart@canonical.com>
 # Author: Alejandro J. Cura <alecu@canonical.com>
 #
-# Copyright 2010, 2011 Canonical Ltd.
+# Copyright 2010-2012 Canonical Ltd.
 #
 # This program is free software: you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 3, as published
@@ -16,27 +16,40 @@
 #
 # You should have received a copy of the GNU General Public License along
 # with this program.  If not, see <http://www.gnu.org/licenses/>.
-"""Single Sign On account management."""
+#
+# In addition, as a special exception, the copyright holders give
+# permission to link the code of portions of this program with the
+# OpenSSL library under certain conditions as described in each
+# individual source file, and distribute linked combinations
+# including the two.
+# You must obey the GNU General Public License in all respects
+# for all of the code used other than OpenSSL.  If you modify
+# file(s) with this exception, you may extend this exception to your
+# version of the file(s), but you are not obligated to do so.  If you
+# do not wish to do so, delete this exception statement from your
+# version.  If you delete this exception statement from all source
+# files in the program, then also delete it here.
+"""Single Sign On account management.
+
+All the methods in Account expect unicode as parameters.
+
+"""
 
 import os
 import re
-import urllib2
 
-# Unable to import 'lazr.restfulclient.*'
-# pylint: disable=F0401
-from lazr.restfulclient.authorize import BasicHttpAuthorizer
 from lazr.restfulclient.authorize.oauth import OAuthAuthorizer
-from lazr.restfulclient.errors import HTTPError
-from lazr.restfulclient.resource import ServiceRoot
-# pylint: enable=F0401
 from oauth import oauth
+from twisted.internet import defer
 
 from ubuntu_sso.logger import setup_logging
-from ubuntu_sso.utils import timestamp_checker
+from ubuntu_sso.utils import webclient
+from ubuntu_sso.utils.webclient import restful
+from ubuntu_sso.utils.webclient.common import WebClientError
 
 
 logger = setup_logging("ubuntu_sso.account")
-SERVICE_URL = "https://login.ubuntu.com/api/1.0"
+SERVICE_URL = u"https://login.ubuntu.com/api/1.0/"
 SSO_STATUS_OK = 'ok'
 SSO_STATUS_ERROR = 'error'
 
@@ -96,17 +109,15 @@ class NewPasswordError(Exception):
 class Account(object):
     """Login and register users using the Ubuntu Single Sign On service."""
 
-    def __init__(self, sso_service_class=None):
+    def __init__(self, service_url=None):
         """Create a new SSO Account manager."""
-        if sso_service_class is None:
-            self.sso_service_class = ServiceRoot
+        if service_url is not None:
+            self.service_url = service_url
         else:
-            self.sso_service_class = sso_service_class
-
-        self.service_url = os.environ.get('USSOC_SERVICE_URL', SERVICE_URL)
-
-        logger.info('Created a new SSO access layer for service url %r',
+            self.service_url = os.environ.get('USSOC_SERVICE_URL', SERVICE_URL)
+        logger.info('Creating a new SSO access layer for service url %r',
                      self.service_url)
+        assert self.service_url.endswith("/")
 
     def _valid_email(self, email):
         """Validate the given email."""
@@ -130,46 +141,57 @@ class Account(object):
                 result[key] = "\n".join(val)
         return result
 
+    @defer.inlineCallbacks
     def generate_captcha(self, filename):
         """Generate a captcha using the SSO service."""
         logger.debug('generate_captcha: requesting captcha, filename: %r',
                      filename)
-        sso_service = self.sso_service_class(None, self.service_url)
-        captcha = sso_service.captchas.new()
+        restful_client = restful.RestfulClient(self.service_url)
+        try:
+            captcha = yield restful_client.restcall(u"captchas.new")
+        finally:
+            restful_client.shutdown()
 
         # download captcha and save to 'filename'
         logger.debug('generate_captcha: server answered: %r', captcha)
+        wc = webclient.webclient_factory()
         try:
-            res = urllib2.urlopen(captcha['image_url'])
+            response = yield wc.request(captcha['image_url'])
             with open(filename, 'wb') as f:
-                f.write(res.read())
+                f.write(response.content)
         except:
             msg = 'generate_captcha crashed while downloading the image.'
             logger.exception(msg)
             raise
+        finally:
+            wc.shutdown()
 
-        return captcha['captcha_id']
+        defer.returnValue(captcha['captcha_id'])
 
+    @defer.inlineCallbacks
     def register_user(self, email, password, displayname,
                       captcha_id, captcha_solution):
         """Register a new user with 'email' and 'password'."""
         logger.debug('register_user: email: %r password: <hidden>, '
                      'displayname: %r, captcha_id: %r, captcha_solution: %r',
                      email, displayname, captcha_id, captcha_solution)
-        sso_service = self.sso_service_class(None, self.service_url)
-        if not self._valid_email(email):
-            logger.error('register_user: InvalidEmailError for email: %r',
-                         email)
-            raise InvalidEmailError()
-        if not self._valid_password(password):
-            logger.error('register_user: InvalidPasswordError')
-            raise InvalidPasswordError()
+        restful_client = restful.RestfulClient(self.service_url)
+        try:
+            if not self._valid_email(email):
+                logger.error('register_user: InvalidEmailError for email: %r',
+                             email)
+                raise InvalidEmailError()
+            if not self._valid_password(password):
+                logger.error('register_user: InvalidPasswordError')
+                raise InvalidPasswordError()
 
-        result = sso_service.registrations.register(
-                    email=email, password=password,
-                    displayname=displayname,
-                    captcha_id=captcha_id,
-                    captcha_solution=captcha_solution)
+            result = yield restful_client.restcall(u"registration.register",
+                        email=email, password=password,
+                        displayname=displayname,
+                        captcha_id=captcha_id,
+                        captcha_solution=captcha_solution)
+        finally:
+            restful_client.shutdown()
         logger.info('register_user: email: %r result: %r', email, result)
 
         if result['status'].lower() == SSO_STATUS_ERROR:
@@ -178,87 +200,91 @@ class Account(object):
         elif result['status'].lower() != SSO_STATUS_OK:
             raise RegistrationError('Received unknown status: %s' % result)
         else:
-            return email
+            defer.returnValue(email)
 
+    @defer.inlineCallbacks
     def login(self, email, password, token_name):
         """Login a user with 'email' and 'password'."""
         logger.debug('login: email: %r password: <hidden>, token_name: %r',
                      email, token_name)
-        basic = BasicHttpAuthorizer(email, password)
-        sso_service = self.sso_service_class(basic, self.service_url)
-        service = sso_service.authentications.authenticate
-
+        restful_client = restful.RestfulClient(self.service_url,
+                                               username=email,
+                                               password=password)
         try:
-            credentials = service(token_name=token_name)
-        except HTTPError:
+            credentials = yield restful_client.restcall(
+                        u"authentications.authenticate",
+                        token_name=token_name)
+        except WebClientError:
             logger.exception('login failed with:')
             raise AuthenticationError()
+        finally:
+            restful_client.shutdown()
 
         logger.debug('login: authentication successful! consumer_key: %r, ' \
                      'token_name: %r', credentials['consumer_key'], token_name)
-        return credentials
+        defer.returnValue(credentials)
 
-    def is_validated(self, token, sso_service=None):
+    @defer.inlineCallbacks
+    def is_validated(self, token):
         """Return if user with 'email' and 'password' is validated."""
         logger.debug('is_validated: requesting accounts.me() info.')
-        if sso_service is None:
-            oauth_token = oauth.OAuthToken(token['token'],
-                                           token['token_secret'])
-            authorizer = TimestampedAuthorizer(
-                                        timestamp_checker.get_faithful_time,
-                                        token['consumer_key'],
-                                        token['consumer_secret'],
-                                        oauth_token)
-            sso_service = self.sso_service_class(authorizer, self.service_url)
-
-        me_info = sso_service.accounts.me()
+        restful_client = restful.RestfulClient(self.service_url,
+                                               oauth_credentials=token)
+        try:
+            me_info = yield restful_client.restcall(u"accounts.me")
+        finally:
+            restful_client.shutdown()
         key = 'preferred_email'
         result = key in me_info and me_info[key] != None
 
         logger.info('is_validated: consumer_key: %r, result: %r.',
                     token['consumer_key'], result)
-        return result
+        defer.returnValue(result)
 
+    @defer.inlineCallbacks
     def validate_email(self, email, password, email_token, token_name):
         """Validate an email token for user with 'email' and 'password'."""
         logger.debug('validate_email: email: %r password: <hidden>, '
                      'email_token: %r, token_name: %r.',
                      email, email_token, token_name)
-        token = self.login(email=email, password=password,
-                           token_name=token_name)
-
-        oauth_token = oauth.OAuthToken(token['token'], token['token_secret'])
-        authorizer = TimestampedAuthorizer(timestamp_checker.get_faithful_time,
-                                           token['consumer_key'],
-                                           token['consumer_secret'],
-                                           oauth_token)
-        sso_service = self.sso_service_class(authorizer, self.service_url)
-        result = sso_service.accounts.validate_email(email_token=email_token)
+        credentials = yield self.login(email=email, password=password,
+                                       token_name=token_name)
+        restful_client = restful.RestfulClient(self.service_url,
+                                               oauth_credentials=credentials)
+        try:
+            result = yield restful_client.restcall(u"accounts.validate_email",
+                                                   email_token=email_token)
+        finally:
+            restful_client.shutdown()
         logger.info('validate_email: email: %r result: %r', email, result)
         if 'errors' in result:
             errorsdict = self._format_webservice_errors(result['errors'])
             raise EmailTokenError(errorsdict)
         elif 'email' in result:
-            return token
+            defer.returnValue(credentials)
         else:
             raise EmailTokenError('Received invalid reply: %s' % result)
 
+    @defer.inlineCallbacks
     def request_password_reset_token(self, email):
         """Request a token to reset the password for the account 'email'."""
-        sso_service = self.sso_service_class(None, self.service_url)
-        service = sso_service.registrations.request_password_reset_token
+        restful_client = restful.RestfulClient(self.service_url)
         try:
-            result = service(email=email)
-        except HTTPError, e:
+            operation = u"registration.request_password_reset_token"
+            result = yield restful_client.restcall(operation, email=email)
+        except WebClientError, e:
             logger.exception('request_password_reset_token failed with:')
-            raise ResetPasswordTokenError(e.content.split('\n')[0])
+            raise ResetPasswordTokenError(e[1].split('\n')[0])
+        finally:
+            restful_client.shutdown()
 
         if result['status'].lower() == SSO_STATUS_OK:
-            return email
+            defer.returnValue(email)
         else:
             raise ResetPasswordTokenError('Received invalid reply: %s' %
                                           result)
 
+    @defer.inlineCallbacks
     def set_new_password(self, email, token, new_password):
         """Set a new password for the account 'email' to be 'new_password'.
 
@@ -266,16 +292,19 @@ class Account(object):
         'request_password_reset_token'.
 
         """
-        sso_service = self.sso_service_class(None, self.service_url)
-        service = sso_service.registrations.set_new_password
+        restful_client = restful.RestfulClient(self.service_url)
         try:
-            result = service(email=email, token=token,
-                             new_password=new_password)
-        except HTTPError, e:
+            result = yield restful_client.restcall(
+                                        u"registration.set_new_password",
+                                        email=email, token=token,
+                                        new_password=new_password)
+        except WebClientError, e:
             logger.exception('set_new_password failed with:')
-            raise NewPasswordError(e.content.split('\n')[0])
+            raise NewPasswordError(e[1].split('\n')[0])
+        finally:
+            restful_client.shutdown()
 
         if result['status'].lower() == SSO_STATUS_OK:
-            return email
+            defer.returnValue(email)
         else:
             raise NewPasswordError('Received invalid reply: %s' % result)
