@@ -4,7 +4,7 @@
 #
 # Author: Alejandro J. Cura <alecu@canonical.com>
 #
-# Copyright 2010 Canonical Ltd.
+# Copyright 2010-2012 Canonical Ltd.
 #
 # This program is free software: you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 3, as published
@@ -17,10 +17,26 @@
 #
 # You should have received a copy of the GNU General Public License along
 # with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+# In addition, as a special exception, the copyright holders give
+# permission to link the code of portions of this program with the
+# OpenSSL library under certain conditions as described in each
+# individual source file, and distribute linked combinations
+# including the two.
+# You must obey the GNU General Public License in all respects
+# for all of the code used other than OpenSSL.  If you modify
+# file(s) with this exception, you may extend this exception to your
+# version of the file(s), but you are not obligated to do so.  If you
+# do not wish to do so, delete this exception statement from your
+# version.  If you delete this exception statement from all source
+# files in the program, then also delete it here.
 """Implementation of network state detection."""
 
 import dbus
 
+from twisted.internet import defer
+
+from ubuntu_sso.networkstate import NetworkFailException
 from ubuntu_sso.logger import setup_logging
 logger = setup_logging("ubuntu_sso.networkstate")
 
@@ -75,8 +91,6 @@ class NetworkManagerState(object):
 
     def call_result_cb(self, state):
         """Return the state thru the result callback."""
-        if self.state_signal:
-            self.state_signal.remove()
         self.result_cb(state)
 
     def got_state(self, state):
@@ -93,7 +107,9 @@ class NetworkManagerState(object):
         if isinstance(error, self.dbus.exceptions.DBusException) and \
                 error.get_dbus_name() == DBUS_UNKNOWN_SERVICE:
             logger.debug("Network Manager not present")
-            self.call_result_cb(UNKNOWN)
+            # Assuming since Network Manager is not installed,
+            # the user has connected in some other way
+            self.call_result_cb(ONLINE)
         else:
             logger.error("Error contacting NetworkManager: %s" % \
                              str(error))
@@ -125,3 +141,24 @@ class NetworkManagerState(object):
                          error_handler=self.got_error)
         except Exception, e:  # pylint: disable=W0703
             self.got_error(e)
+
+
+def is_machine_connected():
+    """Return a deferred that when fired, returns if the machine is online."""
+    d = defer.Deferred()
+
+    def got_state(state):
+        """The state was retrieved from the Network Manager."""
+        result = int(state) in NM_STATE_CONNECTED_LIST
+        d.callback(result)
+
+    # pylint: disable=W0702, W0703
+    try:
+        network = NetworkManagerState(got_state)
+        network.find_online_state()
+    except Exception, e:
+        logger.exception('is_machine_connected failed with:')
+        d.errback(NetworkFailException(e))
+    # pylint: enable=W0702, W0703
+
+    return d
