@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
-# Author: Manuel de la Pena <manuel@canonical.com>
 #
-# Copyright 2011 Canonical Ltd.
+# Copyright 2011-2012 Canonical Ltd.
 #
 # This program is free software: you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 3, as published
@@ -14,6 +13,19 @@
 #
 # You should have received a copy of the GNU General Public License along
 # with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+# In addition, as a special exception, the copyright holders give
+# permission to link the code of portions of this program with the
+# OpenSSL library under certain conditions as described in each
+# individual source file, and distribute linked combinations
+# including the two.
+# You must obey the GNU General Public License in all respects
+# for all of the code used other than OpenSSL.  If you modify
+# file(s) with this exception, you may extend this exception to your
+# version of the file(s), but you are not obligated to do so.  If you
+# do not wish to do so, delete this exception statement from your
+# version.  If you delete this exception statement from all source
+# files in the program, then also delete it here.
 """Network status implementation on Windows."""
 
 
@@ -25,11 +37,14 @@ import pythoncom
 from ctypes import windll, byref
 from ctypes.wintypes import DWORD
 from threading import Thread
+
+from twisted.internet import defer
 # pylint: disable=F0401
 from win32com.server.policy import DesignatedWrapPolicy
 from win32com.client import Dispatch
 # pylint: enable=F0401
 
+from ubuntu_sso.networkstate import NetworkFailException
 from ubuntu_sso.logger import setup_logging
 
 logger = setup_logging("ubuntu_sso.networkstate")
@@ -162,11 +177,17 @@ class NetworkManager(DesignatedWrapPolicy):
 
 
 def is_machine_connected():
-    """Return if the machine is connected to the internet."""
-    wininet = windll.wininet
-    flags = DWORD()
-    connected = wininet.InternetGetConnectedState(byref(flags), None)
-    return connected == 1
+    """Return a deferred that when fired, returns if the machine is online."""
+    # pylint: disable=W0703, W0702
+    try:
+        wininet = windll.wininet
+        flags = DWORD()
+        connected = wininet.InternetGetConnectedState(byref(flags), None)
+        return defer.succeed(connected == 1)
+    except Exception, e:
+        logger.exception('is_machine_connected failed with:')
+        return defer.fail(NetworkFailException(e))
+    # pylint: enable=W0703, W0702
 
 
 class NetworkManagerState(object):
@@ -198,4 +219,5 @@ class NetworkManagerState(object):
         if listener_thread is None:
             listener_thread = Thread(target=listener.register,
                                      name='Ubuntu SSO NetworkManager')
+        listener_thread.daemon = True
         listener_thread.start()
