@@ -2,7 +2,7 @@
 
 # Author: Alejandro J. Cura <alecu@canonical.com>
 #
-# Copyright 2010 Canonical Ltd.
+# Copyright 2010-2012 Canonical Ltd.
 #
 # This program is free software: you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 3, as published
@@ -15,14 +15,28 @@
 #
 # You should have received a copy of the GNU General Public License along
 # with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+# In addition, as a special exception, the copyright holders give
+# permission to link the code of portions of this program with the
+# OpenSSL library under certain conditions as described in each
+# individual source file, and distribute linked combinations
+# including the two.
+# You must obey the GNU General Public License in all respects
+# for all of the code used other than OpenSSL.  If you modify
+# file(s) with this exception, you may extend this exception to your
+# version of the file(s), but you are not obligated to do so.  If you
+# do not wish to do so, delete this exception statement from your
+# version.  If you delete this exception statement from all source
+# files in the program, then also delete it here.
 """Tests for txkeyring."""
 
+import logging
 import uuid
 
 import dbus.service
 
 from twisted.internet.defer import inlineCallbacks, returnValue
-from ubuntuone.devtools.testcase import DBusTestCase
+from ubuntuone.devtools.testcases.dbus import DBusTestCase
 
 from ubuntu_sso.utils import txsecrets
 
@@ -35,6 +49,9 @@ from ubuntu_sso.utils import txsecrets
 # pylint complains when things are a little too dynamic
 # pylint: disable=E1101
 
+# Use of super on an old style class
+# pylint: disable=E1002
+
 
 ERROR_CREATE_BUT_LOCKED = "Cannot create an item in a locked collection"
 PROMPT_BASE_PATH = "/org/freedesktop/secrets/prompt"
@@ -60,6 +77,7 @@ class ItemMock(dbus.service.Object):
 
     def __init__(self, collection, label, attributes, value, *args, **kwargs):
         """Initialize this instance."""
+        # pylint: disable=E1002
         super(ItemMock, self).__init__(*args, **kwargs)
         self.collection = collection
         self.label = label
@@ -106,6 +124,7 @@ class PromptMock(dbus.service.Object):
     def __init__(self, dismissed=True,
                  result=dbus.String("", variant_level=1), *args, **kwargs):
         """Initialize this instance."""
+        # pylint: disable=E1002
         super(PromptMock, self).__init__(*args, **kwargs)
         self.dismissed = dismissed
         self.result = result
@@ -139,6 +158,7 @@ class BaseCollectionMock(dbus.service.Object):
 
     def __init__(self, label, *args, **kwargs):
         """Initialize this instance."""
+        # pylint: disable=E1002
         super(BaseCollectionMock, self).__init__(*args, **kwargs)
         self.items = []
         self.label = label
@@ -217,6 +237,7 @@ class SecretServiceMock(dbus.service.Object):
 
     def __init__(self, *args, **kwargs):
         """Initialize this instance."""
+        # pylint: disable=E1002
         super(SecretServiceMock, self).__init__(*args, **kwargs)
         self.sessions = {}
         self.collections = {}
@@ -376,6 +397,7 @@ class AltSecretServiceMock(SecretServiceMock):
                          in_signature="a{sv}s", out_signature="oo")
     def CreateCollection(self, properties, alias):
         """Create a new collection with the specified properties."""
+        # pylint: disable=E1002
         collection, prompt = super(AltSecretServiceMock,
                                    self).CreateCollection(properties)
         self.SetAlias(alias, collection)
@@ -389,6 +411,19 @@ def create_object_path(base):
     return base + "/" + random
 
 
+class TextFilter(object):
+    """Prevents the logging of messages containing a given text."""
+
+    def __init__(self, *args):
+        """Initialize this filter."""
+        super(TextFilter, self).__init__()
+        self.filtered_strings = args
+
+    def filter(self, record):
+        """See if we need to filter a given log record."""
+        return not any(s in record.msg for s in self.filtered_strings)
+
+
 class BaseTestCase(DBusTestCase):
     """Base class for DBus tests."""
     timeout = 10
@@ -401,6 +436,14 @@ class BaseTestCase(DBusTestCase):
         self.mock_service = self.dbus_publish(txsecrets.SECRETS_SERVICE,
                                               self.secret_service_class)
         self.secretservice = txsecrets.SecretService()
+        self.silence_dbus_logging()
+
+    def silence_dbus_logging(self):
+        """Silence the warnings printed by dbus that pollute test results."""
+        logger = logging.getLogger('dbus.connection')
+        logfilter = TextFilter("Unable to set arguments")
+        logger.addFilter(logfilter)
+        self.addCleanup(logger.removeFilter, logfilter)
 
     def dbus_publish(self, object_path, object_class, *args, **kwargs):
         """Create an object and publish it on the bus."""
